#
# $Id: mcMiniSysInfo.pm,v 1.4 2002/03/09 01:05:34 mcooper Exp $
#
# Minimum SysInfo object
#
# This object is used just to bootstrap the main sysinfo
#

package mcMiniSysInfo;
require 5.003;

use mcMisc;
use mcDebug;

my @UnamePaths = ( '/usr/bin/uname', '/bin/uname' );

#
# Object Constructor
#
sub New {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    my $Err;

    $self->{"OSname"} = undef;
    $self->{"OSversion"} = undef;
    $self->{"OSmajVersion"} = undef;
    $self->{"LibcName"} = undef;
    $self->{"LibcVer"} = undef;
    $self->{"AppArch"} = undef;
    $self->{"CpuArch"} = undef;
    $self->{"Platform"} = undef;

    bless ($self, $class);

    return $self;
}

sub OSname {
    my $self = shift;

    if (!@_) {
	return $self->{"OSname"};
    }

    my $s = lc(shift);
    $s =~ s/[\/-]//g;
    $self->{"OSname"} = $s;

    return $self->{"OSname"};
}

sub OSversion {
    my $self = shift;

    if (!@_) {
	return $self->{"OSversion"};
    }

    my $s = shift;
    $self->{"OSversion"} = $s;

    return $self->{"OSversion"};
}

sub OSmajVersion {
    my $self = shift;

    if (!@_) {
	return $self->{"OSmajVersion"};
    }

    my $s = shift;
    $self->{"OSmajVersion"} = $s;

    return $self->{"OSmajVersion"};
}

sub AppArch {
    my $self = shift;

    if (!@_) {
	return $self->{"AppArch"};
    }

    my $s = shift;
    $self->{"AppArch"} = $s;

    return $self->{"AppArch"};
}

sub CpuArch {
    my $self = shift;

    if (!@_) {
	return $self->{"CpuArch"};
    }

    my $s = shift;
    $self->{"CpuArch"} = $s;

    return $self->{"CpuArch"};
}

sub LibcName {
    my $self = shift;

    if (!@_) {
	return $self->{"LibcName"};
    }

    my $s = shift;
    $self->{"LibcName"} = $s;

    return $self->{"LibcName"};
}

sub LibcVer {
    my $self = shift;

    if (!@_) {
	return $self->{"LibcVer"};
    }

    my $s = shift;
    $self->{"LibcVer"} = $s;

    return $self->{"LibcVer"};
}

sub Platform {
    my $self = shift;

    if (!@_) {
	return $self->{"Platform"};
    }

    my $s = shift;
    $self->{"Platform"} = $s;

    return $self->{"Platform"};
}

sub FindProgram {
    my @Paths = @_;

    foreach my $path (@Paths) {
	if (-x $path) {
	    return $path;
	}
    }
}

sub Command {
    my @Argv = @_;
    my $Cmd = join(' ', @Argv);

    Debug("Running: <%s>", $Cmd);
    my $Result = `$Cmd`;
    chop($Result);

    return $Result;
}

#
# Initialize all data
#
sub Init {
    my $self = shift;

    Debug("Determining minimum system information . . .");

    # Make sure we have a consistant path
    $ENV{'PATH'} = '/bin:/usr/bin:/usr/ucb:/sbin';

    my $uname = FindProgram(@UnamePaths);
    if (!$uname) {
	Error("uname: Unable to find path to command.");
	return;
    }

    if (!$self->OSname(Command($uname, '-s'))) {
	Error("uname: Cannot determine OS name.");
	return;
    }

    #
    # Get OS Version
    #
    if ($self->OSname eq 'aix') {
	my $ver = Command($uname, '-v');
	if ($ver == 4) {
	    my $rev = Command($uname, '-r');
	    $ver = $ver . '.' . $rev;
	}
	$self->OSversion($ver);
    } elsif ($self->OSname eq 'hpux') {
	my $ver = Command($uname, '-r');
	$ver =~ s/^[A-Za-z]\.//;
	$ver =~ s/^[0]//;
	$self->OSversion($ver);
    } elsif ($self->OSname eq 'linux') {
	my $ver = Command($uname, '-r');
	my @p = split(/[\.]/, $ver);
	$self->OSversion(sprintf("%d.%d", $p[0], $p[1]));
    } else {
	$self->OSversion(Command($uname, '-r'));
    }

    #
    # OS Major version
    #
    my @p = split(/[\.]/, $self->OSversion);
    $self->OSmajVersion($p[0]);

    #
    # Get App Arch
    #
    if ($self->OSname eq 'aix') {
	# Hard code since we don't know how else to get this
	$self->AppArch('powerpc');
    } elsif ($self->OSname eq 'hpux') {
	my $arch = Command($uname, '-m');
	$arch =~ s/\/.*//;
	$arch =~ s/9000/parisc/;
	$self->AppArch($arch);
    } elsif ($self->OSname eq 'linux') {
	my $arch = Command($uname, '-m');
	# Special hack for AMD systems
	if ($arch eq 'i586') {
	    $arch = 'i686';
	}
	$self->AppArch($arch);
    } elsif ($self->OSname eq 'sunos') {
	if ($self->OSmajVersion == 4) {
	    $self->AppArch(Command('/bin/mach'));
	} else {
	    $self->AppArch(Command($uname, '-p'));
	}
    } elsif ($self->OSname =~ /irix/) {
	$self->AppArch(Command($uname, '-p'));
    } else {
	$self->AppArch(Command($uname, '-m'));
    }

    #
    # Get CPU Architecture
    #
    if ($self->OSname eq 'sunos') {
	if ($self->OSmajVersion == 5) {
	    if (-x '/bin/isainfo') {
		$self->CpuArch(Command('/bin/isainfo', '-k'));
	    } else {
		$self->CpuArch(Command($uname, '-p'));
	    }
	} else {
	    $self->CpuArch(Command($uname, '-m'));
	}
    } else {
	$self->CpuArch($self->AppArch);
    }

    #
    # Libc Name and version
    #
    if ($self->OSname eq 'linux') {
	my $Libc = Command('ldd /bin/sh | grep libc.so | awk \'{print $3}\'');
	my $LibcVerStr = Command("strings $Libc | grep -i \'c library\'");
	$LibcVerStr =~ s/,/ /g;
	my @Fields = split(/[ ]+/, $LibcVerStr);
	$self->LibcVer($Fields[6]);
	if ($LibcVerStr =~ /GNU/) {
	    $self->LibcName('glibc');
	} else {
	    $self->LibcName('libc');
	}
    }

    return $self;
}

1; # Make require/use happy
