/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.5 $";
#endif

/*
 * Printer LP (System V) functions
 *
 * This works on most "lp" systems which provide lpstat.
 * While this is fairly portable, the data retrieved is not
 * very rich due to the limited data provided by lpstat.
 */

#include "defs.h"
#if HAVE_SETJMP_H
# include <setjmp.h>
static jmp_buf			Env;
#endif

static char *LpstatPath[] = {
    "/usr/bin/lpstat", "/bin/lpstat", NULL
};

/*
 * Called when the alarm goes off
 */
static void TimeOutHandler(SigNum)
     int			SigNum;
{
    if (SigNum == SIGALRM)
	SImsg(SIM_DBG,
	      "TimeOutHandler: A time-out has occured executing a comamnd.");
    else
	SImsg(SIM_DBG,
	      "TimeOutHandler: Cmd Received signal %s",
	      SignalName(SigNum));

#if	defined(HAVE_LONGJMP)
    longjmp(Env, SigNum);
#endif	/* HAVE_LONGJMP */
}

/*
 * Use lpstat to find printers
 */
static int PrinterLpstat(PrinterTree, SearchExp)
     Printer_t		      **PrinterTree;
     char		      **SearchExp;
{
    Printer_t		       *Printer;
    Printer_t		       *LastPrinter = NULL;
    char		       *Lpstat;
    static char			Cmd[1024];
    static char			Buff[1024];
    char		       *Key;
    char		       *Val;
    char		       *cp;
    FILE		       *Fp;
    int 			LongJumpVal;

    Lpstat = (char *) CmdFind(LpstatPath);
    if (!Lpstat) {
	SImsg(SIM_DBG, "PrinterLpstat: Cannot find \"lpstat\" command.");
	return -1;
    }

    snprintf(Cmd, sizeof(Cmd), "%s -v", Lpstat);

    SImsg(SIM_DBG, "PrinterLpstat: Looking for printers from <%s>", Cmd);

#if	defined(HAVE_SETJMP)
    /*
     * Set a timeout since programs may hang
     */
    if ((LongJumpVal = setjmp(Env)) != 0) {
	/*
	 * If we reach this point then the time-out has occured
	 */
	cp = SignalName(LongJumpVal);
	SImsg(SIM_DBG, "PrinterLpstat: Command <%s> terminated due to %s", 
	      Cmd, cp);
	return -1;
    }
    TimeOutStart(60, TimeOutHandler);
#endif	/* HAVE_SETJMP */

    if (Debug) {
	/*
	 * Get some raw output for debug
	 */
	SImsg(SIM_DBG, "LPSTAT (%s) BEGIN", Cmd);
	system(Cmd);
	SImsg(SIM_DBG, "LPSTAT (%s) END", Cmd);
    }

    Fp = popen(Cmd, "r");
    if (!Fp) {
	SImsg(SIM_DBG, "PrinterLpstat: Command \"%s\" open failed: %s",
	      Cmd, SYSERR);
	TimeOutEnd();
	return -1;
    }

    /*
     * Output should look something like:
     * 	device for mcp: /dev/null
     *	     remote to: mcp on dune.magnicomp.com 
     */

    while (fgets(Buff, sizeof(Buff), Fp)) {
	if (cp = strchr(Buff, '\n'))
	    *cp = CNULL;

	SImsg(SIM_DBG, "PrinterLpstat: Lpstat Returned <%s>", Buff);

	/* Set Key skipping leading white space */
	for (Key = Buff; Key && *Key && isspace(*Key); ++Key);
	if (cp = strchr(Key, ':')) {
	    *cp = CNULL;
	    /* Set Val skipping whitespace */
	    for (Val = cp + 1; Val && *Val && isspace(*Val); ++Val);
	} else {
	    Val = NULL;
	}

	if (EQN(Key, "device for", 10)) {
	    /* Set cp to the queue name and skip whitespace */
	    for (cp = Key + 10; cp && *cp && isspace(*cp); ++cp);

	    if (strlen(cp) <= 0)
		continue;

	    SImsg(SIM_DBG, "PrinterLpstat: Found Printer <%s>",
		  cp);

	    Printer = LastPrinter = PrinterCreate(NULL);
	    Printer->Queue = strdup(cp);
	    if (Val && !EQ(Val, "/dev/null")) {
		Printer->Device = strdup(Val);
		Printer->Proto = PP_LOCAL;
	    }
	    PrinterAdd(PrinterTree, Printer);
	} else if (EQN(Key, "remote to", 9)) {
	    if (!LastPrinter) {
		SImsg(SIM_DBG, "PrinterLpstat: LastPrinter is not set. Why?");
		continue;
	    }
	    if (!Val)
		continue;

	    Printer->RemoteQueue = strdup(Val);
	    if (cp = strchr(Printer->RemoteQueue, ' ')) {
		*cp = CNULL;
		++cp;
	    } else {
		continue;
	    }

	    if (EQN(cp, "on ", 3))
		cp += 3;

	    /* Skip any remaining white space */
	    for ( ; cp && *cp && isspace(*cp); ++cp);

	    if (cp && strlen(cp)) {
		Printer->Server = strdup(cp);
		Printer->Proto = PP_LPD;	/* XXX Assume it's LPD */
	    }
	}
    }

    TimeOutEnd();
    fclose(Fp);

    return 0;
}

/*
 * Build Printer info from a "stock" LP system.
 */
extern Opaque_t BuildPrinterLp(PrinterTree, SearchExp)
     Printer_t		      **PrinterTree;
     char		      **SearchExp;
{
    int				Found = 0;

    if (!PrinterTree)
	return (Opaque_t) -1;

    Found = PrinterLpstat(PrinterTree);

    return (Opaque_t) Found;
}
