#
# $Id: uiDataDetails.pm,v 1.6 2002/03/19 17:32:28 mcooper Exp $
#
# UI Data Details functions
#

package uiDataDetails;

require 5.003;

require Exporter;

use mcDebug;
use uiMisc;
use uiGlobals;
use uiStyles;

use Tk;
use Tk::HList;

#
# Variables
#

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   DataDetails
		  );
}

#
# Show the details of a data entry in a new popup window.
#
sub DataDetails {
    my $Args = shift;
    my $Path = $Args->{Path};
    my $Table = $Args->{Table};
    my $TreePathSep = $Args->{TreePathSep};
    my @Headers = ( 'DESCRIPTION', 'VALUE' );

    my @PropPaths;
    my @PathParts = split($TreePathSep, $Path);
    foreach my $path (sort keys %$Table) {
	my @pathparts = split($TreePathSep, $path);
	if ($#pathparts <= $#PathParts) {
	    next;
	}
	splice @pathparts, $#PathParts + 1;
	my $parentpath = join($TreePathSep, @pathparts);
	if ($Path =~ $parentpath) {
	    Debug("DataDetails: ADD <%s>=~<%s>", $Path, $path);
	    push(@PropPaths, $path);
	}
    }

    if ($#PropPaths < 0) {
	Debug("DataDetails: Path=<%s> Cannot find any props.", $Path);
	return;
    }
    Debug("DataDetails: Path=<%s> Found %d Props.", $Path, $#PropPaths);

    #
    # Calculate the Max lengths of description and value so we can
    # determine an appropriate window width.
    #
    my $MaxDesc;
    my $MaxVal;
    foreach my $path (@PropPaths) {
	my $Value = GetValue($Table->{$path}->{Value});
	if (!$Value || !length($Value)) {
	    next;
	}

	my $Len = length($Value);
	if ($Len > $MaxVal) {
	    $MaxVal = $Len;
	}
	my $Desc = $Table->{$path}->{Desc};
	my $Len = length($Desc);
	if ($Len > $MaxDesc) {
	    $MaxDesc = $Len;
	}
    }
    my $MaxWidth = $MaxDesc + $MaxVal;
    if ($MaxWidth > 80) {
	# Insure our width isn't too large
	$MaxWidth = 80;
    }
    Debug("DataDetails: MaxDesc=%d MaxVal=%d MaxWidth=%d",
	  $MaxDesc, $MaxVal, $MaxWidth);

    my @p = split($TreePathSep, $Path);
    my $Name = $p[$#p];

    my $win = MainWindow->new(-title=>WinTitle(sprintf("Details of %s",
						       $Name)));

    my $Fr = $win->Frame(-borderwidth=>5,
			 -relief=>'groove')
	->pack(-side=>'top', -anchor=>'center', -expand=>1, -fill=>'both');

    my $HList = $Fr->Scrolled("HList",
			      -scrollbars=>'osoe',
			      -columns=>$#Headers + 1,
			      -header=>1,
			      -indent=>10,
			      -separator=>'^',	# Something to disable this
			      -drawbranch=>0,
			      -font=>$FontTree,
			      -height=>10,	# in chars
			      -width=>$MaxWidth,# in chars
			      -background=>'white',
			     )
	->pack(-side=>'top', -anchor=>'center', -expand=>1, -fill=>'both');

    #
    # Add the headers
    #
    for (my $i = 0; $i <= $#Headers; ++$i) {
	my $name = $Headers[$i];
	Debug("Create header: col=%d text=<%s>",
	      $i, $name);

	$HList->headerCreate($i,
			     -text=>$name,
			     -itemtype=>'text',
			     -borderwidth=>3,
			     -relief=>'groove',
			     #-headerbackground=>'blue',
			    );
    }
    $win->update;

    my $Styles = StylesAdd({Widget=>$HList});

    #
    # Add the actual props
    #
    my $i;
    foreach my $path (@PropPaths) {
	my $Value = GetValue($Table->{$path}->{Value});
	if (!$Value || !length($Value)) {
	    next;
	}
	my $Desc = Casify($Table->{$path}->{Desc});

	my $Style =  $Styles->{odd}->{text};
	if ($i++ % 2) {
	    $Style = $Styles->{even}->{text};
	}

	$HList->add($path);
	$HList->itemCreate($path, 0,
			   -itemtype=>'text',
			   -style=>$Style,
			   -text=>$Desc);
	$HList->itemCreate($path, 1,
			   -itemtype=>'text',
			   -style=>$Style,
			   -text=>$Value);
    }

    #
    # Buttons
    #
    my $CloseBtn = $win->Button(-text=>'Close',
				-font=>$FontButton,
				-command=>[ $win => 'withdraw' ])
	->pack(-side=>'bottom', -anchor=>'center');
}

1;
