#
# $Id: uiKVF.pm,v 1.4 2002/03/10 20:37:09 mcooper Exp $
#
# KVF functions
#

package uiKVF;
require 5.003;
require Exporter;

use mcDebug;
use uiMisc;

BEGIN {
    #
    # Export function
    #
    @ISA      = qw(Exporter);

    #
    # Variables and functions being auto exported
    #
    @EXPORT   = qw(
		   KVFread KVFsave
		  );
}

#
# Read and parse specified Key=Value file
#
sub KVFread {
    my $Args = shift;
    my $File = $Args->{File};
    my $QuietError = $Args->{QuietError};
    my $KV;

    Debug("Reading KV File <%s>", $File);

    my $FH = GetFH();
    if (!open($FH, "< $File")) {
	if (!$QuietError || $Debug) {
	    Error("%s: Open for read failed: %s", $File, $!);
	}
	return;
    }

    my $LineNo = 0;
    while (my $Line = <$FH>) {
	++$LineNo;
	chop($Line);
	$Line =~ s/\#.*//;	# Remove comments
	$Line =~ s/[ \t]//g;	# Remove whitespace
	if ($Line !~ /=/) {
	    next;
	}
	my @Fields = split('=', $Line);
	if ($#Fields != 1) {
	    Error("%s: Line %d: Wrong number of fields (expecting 2, found %d).",
		  $File, $LineNo, $#Fields+1);
	    next;
	}
	my $Key = $Fields[0];
	my $Value = $Fields[1];
	$Value =~ s/\"//g; 	# Remove quotes
	$KV->{$Key} = $Value;
    }

    close($FH);

    if ($Debug) {
	foreach my $key (sort keys %$KV) {
	    Debug("KV <%s>=<%s>", $key, $KV->{$key});
	}
    }

    return $KV;
}

#
# Save a Key=Value File
#
sub KVFsave {
    my $Args = shift;
    my $File = $Args->{File};
    my $KV = $Args->{KV};

    Debug("KVFsave: File=<%s>", $File);

    if (!$File) {
	Error("KVFsave: No file name specified.");
	return -1;
    }

    my $FH = GetFH();
    if (!open($FH, "> $File")) {
	Error("%s: Open for write failed: %s", $File, $!);
	return -1;
    }

    foreach my $key (sort keys %$KV) {
	printf $FH ("%s=%s\n", $key, $KV->{$key});
    }

    close($FH);

    return 0;
}

1;
