/*
 * Copyright (c) 1992-2002 MagniComp
 * This software may only be used in accordance with the license which is
 * available as http://www.magnicomp.com/sysinfo/4.2/sysinfo-eu-license.shtml
 */

#ifndef lint
static char *RCSid = "$Revision: 1.8 $";
#endif

/*
 * Message functions
 */

#include <stdio.h>
#include <string.h>
#include "options.h"
#include "defs.h"

static char			Buff[BUFSIZ];
char			       *LogFile = NULL;
extern void			Log();
static int			NeedNL;

#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
/*
 * StdArg version of Msg()
 */
extern void Msg(int MsgFlags, char *fmt, ...)
{
    va_list 			args;

    if (FLAGS_ON(MsgFlags, MSGF_FILE)) {
	DisplayFile(fmt);
	return;
    }

    if (strchr(fmt, '\n'))
	NeedNL = 0;
    else
	NeedNL = 1;

    va_start(args, fmt);
#if	defined(HAVE_VSNPRINTF)
    (void) vsnprintf(Buff, sizeof(Buff), fmt, args);
#else
    (void) vsprintf(Buff, fmt, args);
#endif
    va_end(args);

    printf("%s", Buff);
    fflush(stdout);

    if (FLAGS_OFF(MsgFlags, MSGF_NOLOG))
	Log(Buff);
}
#endif	/* ARG_STDARG */
#if	ARG_TYPE == ARG_VARARG
#include <varargs.h>
/*
 * Varargs version of Msg()
 */
extern void Msg(MsgFlags, fmt, va_alist)
     int			MsgFlags;
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    if (FLAGS_ON(MsgFlags, MSGF_FILE)) {
	DisplayFile(fmt);
	return;
    }

    if (strchr(fmt, '\n'))
	NeedNL = 0;
    else
	NeedNL = 1;

    va_start(args);
#if	defined(HAVE_VSNPRINTF)
    (void) vsnprintf(Buff, sizeof(Buff), fmt, args);
#else
    (void) vsprintf(Buff, fmt, args);
#endif
    va_end(args);
    printf("%s", Buff);
    fflush(stdout);

    if (FLAGS_OFF(MsgFlags, MSGF_NOLOG))
	Log(Buff);
}
#endif	/* ARG_VARARG */
#if	!defined(ARG_TYPE) || ARG_TYPE == ARG_NONE
/*
 * Non-Varargs version of Msg()
 */
extern void Msg(MsgFlags, fmt, a1, a2, a3, a4, a5, a6)
     int			MsgFlags;
     char 		       *fmt;
{
    if (FLAGS_ON(MsgFlags, MSGF_FILE)) {
	DisplayFile(fmt);
	return;
    }

    if (strchr(fmt, '\n'))
	NeedNL = 0;
    else
	NeedNL = 1;

    (void) snprintf(Buff, sizeof(Buff), fmt, a1, a2, a3, a4, a5, a6);
    printf("%s", Buff);
    fflush(stdout);

    if (FLAGS_OFF(MsgFlags, MSGF_NOLOG))
	Log(Buff);
}
#endif 	/* !ARG_TYPE */

#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
/*
 * StdArg version of Debug()
 */
extern void Debug(char *fmt, ...)
{
    va_list 			args;

    if (!DebugFlag)
	return;

    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    va_start(args, fmt);
    (void) vfprintf(stderr, fmt, args);
    (void) fprintf(stderr, "\n");
    va_end(args);
}
#endif	/* ARG_STDARG */
#if	ARG_TYPE == ARG_VARARG
#include <varargs.h>
/*
 * Varargs version of Debug()
 */
extern void Debug(fmt, va_alist)
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    if (!DebugFlag)
	return;

    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    va_start(args);
    (void) vfprintf(stderr, fmt, args);
    (void) fprintf(stderr, "\n");
    va_end(args);
}
#endif	/* ARG_VARARG */
#if	!defined(ARG_TYPE) || ARG_TYPE == ARG_NONE
/*
 * Non-Varargs version of Debug()
 */
extern void Debug(fmt, a1, a2, a3, a4, a5, a6)
     char 		       *fmt;
{
    if (!DebugFlag)
	return;

    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    (void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
    (void) fprintf(stderr, "\n");
}
#endif 	/* !ARG_TYPE */

/*
 * Error functions
 */
#if	ARG_TYPE == ARG_STDARG
#include <stdarg.h>
/*
 * StdArg version of Error()
 */
extern void Error(char *fmt, ...)
{
    va_list 			args;

    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    va_start(args, fmt);
    (void) fprintf(stderr, "%s: ", ProgramName);
    (void) vfprintf(stderr, fmt, args);
    (void) fprintf(stderr, "\n");
    va_end(args);
}
#endif	/* ARG_STDARG */
#if	ARG_TYPE == ARG_VARARG
#include <varargs.h>
/*
 * Varargs version of Error()
 */
extern void Error(fmt, va_alist)
     char		       *fmt;
     va_dcl
{
    va_list 			args;

    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    va_start(args);
    (void) fprintf(stderr, "%s: ", ProgramName);
    (void) vfprintf(stderr, fmt, args);
    (void) fprintf(stderr, "\n");
    va_end(args);
}
#endif	/* ARG_VARARG */
#if	!defined(ARG_TYPE) || ARG_TYPE == ARG_NONE
/*
 * Non-Varargs version of Error()
 */
extern void Error(fmt, a1, a2, a3, a4, a5, a6)
     char 		       *fmt;
{
    if (NeedNL) {
	fprintf(stderr, "\n");
	NeedNL = 0;
    }

    (void) fprintf(stderr, "%s: ", ProgramName);
    (void) vfprintf(stderr, fmt, a1, a2, a3, a4, a5, a6);
    (void) fprintf(stderr, "\n");
}
#endif 	/* !ARG_TYPE */

/*
 * Set name of log file
 */
extern void SetLogFile(File)
     char		       *File;
{
    if (File)
	LogFile = File;
}

/*
 * Log a message to the file
 */
extern void Log(Msg)
     char		       *Msg;
{
    static FILE		       *FH;

    if (!FH) {
	if (!LogFile) {
	    if (DebugFlag)
		Error("Log(): No LogFile has been set.");
	    return;
	}
	FH = fopen(LogFile, "a");
	if (!FH) {
	    Error("%s: Open log file for write failed: %s", LogFile, SYSERR);
	    return;
	}

	if (fchmod(fileno(FH), 00444) != 0) {
	    Error("%s: Chmod failed: %s", LogFile, SYSERR);
	    return;
	}
    }

    fprintf(FH, "%s", Msg);
}

/*
 * Display a file to the user
 */
extern int DisplayFile(File)
     char		       *File;
{
    static char			Line[8192];
    FILE		       *fp;

    fp = fopen(File, "r");
    if (!fp) {
	Error("%s: File open for read failed: %s", File, SYSERR);
	return -1;
    }

    while (fgets(Line, sizeof(Line), fp)) {
	printf("%s", Line);
	Log(Line);
    }

    fclose(fp);

    return 0;
}

/*
 * Get input from user via terminal
 */
extern char *GetInput(MsgFlags)
     int			MsgFlags;
{
    static char			Input[8192];
    char		       *cp;

    if (fgets(Input, sizeof(Input), stdin)) {
	if (FLAGS_OFF(MsgFlags, MSGF_NOLOG))
	    Log(Input);
	if (cp = strchr(Input, '\n')) {
	    NeedNL = 0;
	    *cp = CNULL;
	}
	return Input;
    }

    Debug("GetInput(): fgets stdin failed: %s", SYSERR);
    return (char *) NULL;
}

/*
 * Show the message contained in File
 */
extern int MsgShowFile(File, Info)
     char		       *File;
     McdInfo_t		       *Info;
{
    static char			Path[MAXPATHLEN];
    static char			Line[512];
    FILE		       *FH;

    if (!Info || !Info->InstallDir)
	return -1;

    (void) snprintf(Path, sizeof(Path), "%s/%s", 
		    Info->InstallDir, File);

    FH = fopen(Path, "r");
    if (!FH) {
	Debug("%s: Open failed: %s", Path, SYSERR);
	return 0;
    }

    while (fgets(Line, sizeof(Line), FH)) {
	Msg(0, "%s", Line);
    }

    fclose(FH);

    return 0;
}
